from windchill_metric_config.description import Description


class Memory:
    def __init__(self):
        self.heap_threshold = Description(
            metric_id='windchill_server_status_memory_heap_usage_threshold_percent',
            desc='Heap memory usage threshold in percent',
            labels=['server_type', 'sub_server_types']
        )
        self.heap_percent = Description(
            metric_id='windchill_server_status_memory_heap_usage_percent',
            desc='Heap memory usage in percent',
            labels=['server_type', 'sub_server_types']
        )
        self.perm_gen_threshold = Description(
            metric_id='windchill_server_status_memory_perm_gen_usage_threshold_percent',
            desc='Perm gen memory usage threshold in percent',
            labels=['server_type', 'sub_server_types']
        )
        self.perm_gem_percent = Description(
            metric_id='windchill_server_status_memory_perm_gen_usage_percent',
            desc='Perm gen memory usage in percent',
            labels=['server_type', 'sub_server_types']
        )

    def as_dict(self):
        all_metrics = {}
        for item in self.__dict__.keys():
            all_metrics[item] = self.__getattribute__(item).as_dict()
        return all_metrics

    def as_yaml_dict(self):
        metrics = {}
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            if type(child) == Description:
                metrics[child.id] = child.enabled
            else:
                metrics[item] = child.as_yaml_dict()
        return metrics

    def as_treeview(self):
        metrics = []
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            tree_item = {}
            if type(child) == Description:
                tree_item['text'] = child.id
                tree_item['state'] = {'checked': child.enabled}
                tree_item['data'] = {'description': child.description}
            else:
                tree_item['text'] = item
                tree_item['children'] = child.as_treeview()
            metrics.append(tree_item)
        return metrics

    def generate_yaml(self, yaml_object, comment_indent):
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            if type(child) == Description:
                yaml_object.yaml_add_eol_comment(child.description, child.id,
                                                 comment_indent)
            else:
                child.generate_yaml(yaml_object[item], comment_indent)

    def metrics_as_list(self, metric_list: list):
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            if type(child) == Description:
                metric_list.append(child)
            else:
                child.metrics_as_list(metric_list)

    def set_config(self, config: dict):
        for key in config:
            for item in self.__dict__.keys():
                child = self.__getattribute__(item)
                if type(child) == Description:
                    if child.id == key:
                        child.enabled = config[key]

                else:
                    if item == key:
                        child.set_config(config[key])



from json import dumps

from windchill_metric_config.description import Description
from windchill_metric_config.system.network import Network


class SystemMetrics:
    def __init__(self):
        self.memory_total_bytes = Description(
            metric_id='process_real_memory_total_bytes',
            desc='system total memory in bytes'
        )
        self.memory_used_bytes = Description(
            metric_id='process_real_memory_used_bytes',
            desc='system used memory in bytes'
        )
        self.cpu_usage = Description(
            metric_id='process_cpu_usage_percent',
            desc='cpu utilisation in percent'
        )
        self.disc_total = Description(
            metric_id='process_disc_total_bytes',
            desc='total disc space in bytes',
            labels=['mount_point']
        )
        self.disc_used = Description(
            metric_id='process_disc_used_bytes',
            desc='used disc space in bytes',
            labels=['mount_point']
        )
        self.users_count = Description(
            metric_id='process_users_total_count',
            desc='count of logged in users on os',
        )
        self.system_boot = Description(
            metric_id='system_boot_timestamp',
            desc='boot timestamp as label',
            labels=['boot_timestamp'],
            prometheus_method='info'
        )
        self.system_stats = Description(
            metric_id='system_stats_info',
            desc='various information about the host system, '
                 'like os version or processor info',
            labels=['system', 'version', 'processor', 'node',
                    'physical_cpu_count', 'release', 'logical_cpu_count',
                    'machine', 'fqdn'],
            prometheus_method='info'
        )
        self.network = Network()

    def __str__(self):
        return dumps(self.as_dict())

    def as_dict(self):
        all_metrics = {}
        for item in self.__dict__.keys():
            all_metrics[item] = self.__getattribute__(item).as_dict()
        return all_metrics

    def as_yaml_dict(self):
        metrics = {}
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            if type(child) == Description:
                metrics[child.id] = child.enabled
            else:
                metrics[item] = child.as_yaml_dict()
        return metrics

    def as_treeview(self):
        metrics = []
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            tree_item = {}
            if type(child) == Description:
                tree_item['text'] = child.id
                tree_item['state'] = {'checked': child.enabled}
                tree_item['data'] = {'description': child.description}
            else:
                tree_item['text'] = item
                tree_item['children'] = child.as_treeview()
            metrics.append(tree_item)
        return metrics

    def generate_yaml(self, yaml_object, comment_indent):
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            if type(child) == Description:
                yaml_object.yaml_add_eol_comment(child.description, child.id,
                                                 comment_indent)
            else:
                child.generate_yaml(yaml_object[item], comment_indent)

    def set_config(self, config: dict):
        for key in config:
            for item in self.__dict__.keys():
                child = self.__getattribute__(item)
                if type(child) == Description:
                    if child.id == key:
                        child.enabled = config[key]

                else:
                    if item == key:
                        child.set_config(config[key])

    def metrics_as_list(self, metric_list: list):
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            if type(child) == Description:
                metric_list.append(child)
            else:
                child.metrics_as_list(metric_list)



from json import dumps

from prometheus_client import Gauge, Info


class Description:
    def __init__(self, metric_id: str, desc: str = 'test',
                 labels: list = None, enabled: bool = False,
                 prometheus_method: str = 'gauge'):
        """
        :param metric_id: metrics identification,
                          example process_real_memory_total_bytes
        :param desc: description for metric id
        :param labels: list of all required labels, default set values are
                        ['host', 'resource_type', 'environment', 'company']
        """
        self.id = metric_id
        self.labels = ['host', 'resource_type', 'environment', 'company']
        if labels is not None:
            for label in labels:
                if label not in self.labels:
                    self.labels.append(label)
        self.description = desc
        self.enabled = enabled
        if 'gauge' == prometheus_method:
            self.prometheus_obj = Gauge(metric_id, desc, self.labels)
        elif 'info' == prometheus_method:
            self.prometheus_obj = Info(metric_id, desc)
            self.prometheus_obj: Info

    def __str__(self):
        return dumps(self.as_dict())

    def as_dict(self):
        obj = {}
        for key in self.__dict__.keys():
            value = self.__getattribute__(key)
            if type(value) is str or type(value) is list or \
                    type(value) is bool:
                obj[key] = value
        return obj



from json import dumps

from windchill_metric_config.description import Description


class Network:
    def __init__(self):
        self.bytes_sent = Description(
            metric_id='process_network_bytes_sent',
            desc='bytes sent over all network addresses',
        )
        self.bytes_recv = Description(
            metric_id='process_network_packets_recv',
            desc='bytes received over all network addresses',
        )
        self.err_in = Description(
            metric_id='process_network_err_in',
            desc='total number of errors while receiving',
        )
        self.err_out = Description(
            metric_id='process_network_err_out',
            desc='total number of errors while sending',
        )
        self.drop_in = Description(
            metric_id='process_network_drop_in',
            desc='total number of incoming packets which were dropped',
        )
        self.drop_out = Description(
            metric_id='process_network_drop_out',
            desc='total number of outgoing packets which were dropped '
                 '(always 0 on macOS and BSD)',
        )

    def __str__(self):
        return dumps(self.as_dict())

    def as_dict(self):
        all_metrics = {}
        for item in self.__dict__.keys():
            all_metrics[item] = self.__getattribute__(item).as_dict()
        return all_metrics

    def as_yaml_dict(self):
        metrics = {}
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            if type(child) == Description:
                metrics[child.id] = child.enabled
            else:
                metrics[item] = child.as_yaml_dict()
        return metrics

    def as_treeview(self):
        metrics = []
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            tree_item = {}
            if type(child) == Description:
                tree_item['text'] = child.id
                tree_item['state'] = {'checked': child.enabled}
                tree_item['data'] = {'description': child.description}
            else:
                tree_item['text'] = item
                tree_item['children'] = child.as_treeview()
            metrics.append(tree_item)
        return metrics

    def generate_yaml(self, yaml_object, comment_indent):
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            if type(child) == Description:
                yaml_object.yaml_add_eol_comment(child.description, child.id,
                                                 comment_indent)
            else:
                child.generate_yaml(yaml_object[item], comment_indent)

    def set_config(self, config: dict):
        for key in config:
            for item in self.__dict__.keys():
                child = self.__getattribute__(item)
                if type(child) == Description:
                    if child.id == key:
                        child.enabled = config[key]

                else:
                    if item == key:
                        child.set_config(config[key])

    def metrics_as_list(self, metric_list: list):
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            if type(child) == Description:
                metric_list.append(child)
            else:
                child.metrics_as_list(metric_list)



from json import dumps, loads

from ruamel.yaml import YAML, scalarstring, CommentedMap

from windchill_metric_config.description import Description
from windchill_metric_config.system.system import SystemMetrics
from windchill_metric_config.windchill.windchill import WindchillMetrics


class Metrics:
    comment_indent = 80

    def __init__(self):
        self.system = SystemMetrics()
        self.windchill = WindchillMetrics()

    def __str__(self):
        return dumps(self.as_dict())

    def as_dict(self):
        return {
            'system': self.system.as_dict(),
            'windchill': self.windchill.as_dict()
        }

    def as_yaml_dict(self):
        return {
            'system': self.system.as_yaml_dict(),
            'windchill': self.windchill.as_yaml_dict()
        }

    def as_treeview(self):
        metrics = []
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            tree_item = {}
            if type(child) == Description:
                tree_item['text'] = child.id
                tree_item['state'] = {'checked': child.enabled}
                tree_item['data'] = {'description': child.description}
            else:
                tree_item['text'] = item
                tree_item['children'] = child.as_treeview()
            metrics.append(tree_item)
        return metrics

    def add_yaml_comments(self, config: dict):
        for key in self.__dict__.keys():
            if key in config:
                item = self.__getattribute__(key)
                if type(item) != Description:
                    item.generate_yaml(config[key], self.comment_indent)

    def save_as_yaml(self, config_yaml: str):
        yaml = YAML()
        data = loads(dumps(self.as_yaml_dict()), object_pairs_hook=CommentedMap)
        scalarstring.walk_tree(data)
        self.windchill.generate_yaml(data['windchill'], self.comment_indent)
        self.system.generate_yaml(data['system'], self.comment_indent)
        with open(config_yaml, 'w', encoding='utf-8') as f:
            yaml.dump(data, f)

    def set_config(self, config: dict):
        for key in self.__dict__.keys():
            if key in config:
                item = self.__getattribute__(key)
                if type(item) != Description:
                    item.set_config(config[key])

    def load_config_from_yaml_file(self, config_yaml: str):
        yaml = YAML()
        with open(config_yaml, 'r', encoding='utf-8') as f:
            data = yaml.load(f)
        for key in self.__dict__.keys():
            if key in data:
                if key == 'system':
                    self.__getattribute__(key).set_config(data[key])

    def metrics_as_list(self) -> [Description]:
        metric_list = []
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            if type(child) == Description:
                metric_list.append(child.id)
            else:
                child.metrics_as_list(metric_list)

        return metric_list

    def metrics_id_list(self) -> [str]:
        metric_list = []
        for metric in self.metrics_as_list():
            metric_list.append(metric.id)
        return metric_list

    def get_metric_config(self, metrics_id: str) -> Description:
        for metric in self.metrics_as_list():
            if metrics_id == metric.id:
                return metric

    def validate_metric(self, post_metric: dict) -> [str]:
        """
        * This is a bulleted list.
        * It has two items, the second item uses two lines.
        This is a normal text paragraph. The next paragraph is a code sample::

           {
              "metric_id": "process_real_memory_total_bytes",
              "labels": [
                {
                  "key": "environment",
                  "value": "anton500"
                }
              ],
              "value": 20.35
            }

        This is a normal text paragraph again.
        :param post_metric: dictionary with metric_id and labels
        :type post_metric dict
        :return: list with all failed labels
        """
        posted_labels = []
        for posted_label_key in post_metric['labels'].keys():
            if len(post_metric['labels'][posted_label_key]) > 0:
                posted_labels.append(posted_label_key)

        metric: Description = self.get_metric_config(post_metric.get(
            'metric_id'))
        error_bucket = []
        for label in metric.labels:
            if label not in posted_labels:
                error_bucket.append(label)
        return error_bucket



from windchill_metric_config.description import Description


def as_yaml_dict(clazz):
    metrics = {}
    for item in clazz.__dict__.keys():
        child = clazz.__getattribute__(item)
        if type(child) == Description:
            metrics[child.id] = child.enabled
        else:
            metrics[item] = child.as_yaml_dict()
    return metrics


def generate_yaml(clazz, yaml_object, comment_indent):
    for item in clazz.__dict__.keys():
        child = clazz.__getattribute__(item)
        if type(child) == Description:
            desc = child.description
            if desc is not 'test' and len(desc) > 0:
                yaml_object.yaml_add_eol_comment(desc, child.id,
                                                 comment_indent)
        else:
            child.generate_yaml(yaml_object[item], comment_indent)



from json import dumps

from windchill_metric_config.description import Description
from windchill_metric_config.windchill.method_server import MethodServer
from windchill_metric_config.windchill.garbage_collector import \
    GarbageCollector
from windchill_metric_config.windchill.memory import Memory
from windchill_metric_config.windchill.method_context import MethodContext
from windchill_metric_config.windchill.queue_worker import QueueWorker


class WindchillMetrics:

    def __init__(self):

        self.apache_status = Description(
            metric_id='windchill_apache_status',
            desc='windchill apache status (0=not running, >1=http code)'
        )
        self.windchill_status = Description(
            metric_id='windchill_windchill_status',
            desc='windchill app status (0=not running, >1=http code)'
        )
        self.api_resp_time = Description(
            metric_id='windchill_api_response_time_seconds',
            desc='windchill api (/Windchill/api/v1/publishmonitor'
                 '/getworkerinfo.jsp) response time'
        )
        self.active_users = Description(
            metric_id='windchill_active_users_total',
            desc='windchill total active users count',
        )
        self.version_info = Description(
            metric_id='windchill_version_info',
            desc='windchill version and release info',
            labels=['release_id', 'sequence', 'release', 'display',
                    'data_code']
        )
        self.version_info_simple = Description(
            metric_id='windchill_version_info_simple',
            desc='windchill version and release info',
            labels=['data_code']
        )
        self.language = Description(
            metric_id='windchill_installed_languages',
            desc='windchill installed languages',
            labels=['locale']
        )

        self.method_server = MethodServer()
        self.queue_worker = QueueWorker()
        self.garbage_collector = GarbageCollector()
        self.memory = Memory()
        self.method_context = MethodContext()

    def __str__(self):
        return dumps(self.as_dict())

    def as_dict(self):
        all_metrics = {}
        for item in self.__dict__.keys():
            all_metrics[item] = self.__getattribute__(item).as_dict()
        return all_metrics

    def as_yaml_dict(self):
        metrics = {}
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            if type(child) == Description:
                metrics[child.id] = child.enabled
            else:
                metrics[item] = child.as_yaml_dict()
        return metrics

    def as_treeview(self):
        metrics = []
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            tree_item = {}
            if type(child) == Description:
                tree_item['text'] = child.id
                tree_item['state'] = {'checked': child.enabled}
                tree_item['data'] = {'description': child.description}
            else:
                tree_item['text'] = item
                tree_item['children'] = child.as_treeview()
            metrics.append(tree_item)
        return metrics

    def generate_yaml(self, yaml_object, comment_indent):
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            if type(child) == Description:
                yaml_object.yaml_add_eol_comment(child.description, child.id,
                                                 comment_indent)
            else:
                child.generate_yaml(yaml_object[item], comment_indent)

    def metrics_as_list(self, metric_list: list):
        for item in self.__dict__.keys():
            child = self.__getattribute__(item)
            if type(child) == Description:
                metric_list.append(child)
            else:
                child.metrics_as_list(metric_list)

    def set_config(self, config: dict):
        for key in config:
            for item in self.__dict__.keys():
                child = self.__getattribute__(item)
                if type(child) == Description:
                    if child.id == key:
                        child.enabled = config[key]

                else:
                    if item == key:
                        child.set_config(config[key])



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="windchill-metric-config",
    version="0.0.20",
    author="Matthias Hippen",
    author_email="hippen@gmx.ch",
    description="Config reader for mimamorisan",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/cax-team/mimamori-san",
    project_urls={
        "Bug Tracker": "https://gitlab.com/cax-team/mimamori-san",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=3.6",
    install_requires=[
        'ruamel.yaml',
        'prometheus-client'
    ]
)
